--
-- Module: 	MUX_32_1 
-- Design: 	TOP_ MUXES
-- VHDL code:	RTL code
--
-- Synthesis	Synopsys FPGA Express ver. 3.3 - Option = Preserve Hierarchy
--
-- Description: Multiplexer 32:1
--
-- Version: 1.0
--
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--  Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
-----------------------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

-- Syntax for Synopsys FPGA Express
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on

entity MUX_32_1 is
    port (
        DATA_I		: in std_logic_vector (31 downto 0); 
        SELECT_I	: in std_logic_vector (4 downto 0);
        DATA_O		: out std_logic
	);
end MUX_32_1;

architecture MUX_32_1_arch of MUX_32_1 is
-- Component Decalrations:
component MUXF7
    port (
    	I0	: in std_logic;
    	I1	: in std_logic;
    	S	: in std_logic;
    	O	: out std_logic
    	);
end component;   	
--
component MUXF8
    port (
    	I0	: in std_logic;
    	I1	: in std_logic;
    	S	: in std_logic;
    	O	: out std_logic
    	);
end component;   	
--
-- Signal Declarations:
signal DATA_MSB_0 : 	std_logic;
signal DATA_LSB_0 : 	std_logic;
signal DATA_MSB_1 : 	std_logic;
signal DATA_LSB_1 : 	std_logic;
signal DATA_MSB	  :	std_logic;
signal DATA_LSB   : 	std_logic;
--
begin
--
-- If synthesis tools supports MUXF7 & MUXF8:
--SELECT_PROCESS: process (SELECT_I, DATA_I)
--begin
--	case SELECT_I is
--		when "00000" => DATA_O <= DATA_I (0);
--		when "00001" => DATA_O <= DATA_I (1);
--		when "00010" => DATA_O <= DATA_I (2);
--		when "00011" => DATA_O <= DATA_I (3);
--		when "00100" => DATA_O <= DATA_I (4);
--		when "00101" => DATA_O <= DATA_I (5);
--		when "00110" => DATA_O <= DATA_I (6);
--		when "00111" => DATA_O <= DATA_I (7);
--		when "01000" => DATA_O <= DATA_I (8);
--		when "01001" => DATA_O <= DATA_I (9);
--		when "01010" => DATA_O <= DATA_I (10);
--		when "01011" => DATA_O <= DATA_I (11);
--		when "01100" => DATA_O <= DATA_I (12);
--		when "01101" => DATA_O <= DATA_I (13);
--		when "01110" => DATA_O <= DATA_I (14);
--		when "01111" => DATA_O <= DATA_I (15);
--		when "10000" => DATA_O <= DATA_I (16);
--		when "10001" => DATA_O <= DATA_I (17);
--		when "10010" => DATA_O <= DATA_I (18);
--		when "10011" => DATA_O <= DATA_I (19);
--		when "10100" => DATA_O <= DATA_I (20);
--		when "10101" => DATA_O <= DATA_I (21);
--		when "10110" => DATA_O <= DATA_I (22);
--		when "10111" => DATA_O <= DATA_I (23);
--		when "11000" => DATA_O <= DATA_I (24);
--		when "11001" => DATA_O <= DATA_I (25);
--		when "11010" => DATA_O <= DATA_I (26);
--		when "11011" => DATA_O <= DATA_I (27);
--		when "11100" => DATA_O <= DATA_I (28);
--		when "11101" => DATA_O <= DATA_I (29);
--		when "11110" => DATA_O <= DATA_I (30);
--		when "11111" => DATA_O <= DATA_I (31);
--		when others => DATA_O <= 'X';
--	end case;
--end process SELECT_PROCESS;
--
-- If synthesis tools DOES NOT support MUXF7 & MUXF8 :
SELECT_PROCESS_LSB_0: process (SELECT_I, DATA_I)
begin
	case SELECT_I (2 downto 0) is
		when "000" => DATA_LSB_0 <= DATA_I (0);
		when "001" => DATA_LSB_0 <= DATA_I (1);
		when "010" => DATA_LSB_0 <= DATA_I (2);
		when "011" => DATA_LSB_0 <= DATA_I (3);
		when "100" => DATA_LSB_0 <= DATA_I (4);
		when "101" => DATA_LSB_0 <= DATA_I (5);
		when "110" => DATA_LSB_0 <= DATA_I (6);
		when "111" => DATA_LSB_0 <= DATA_I (7);
		when others => DATA_LSB_0 <= 'X';
	end case;
end process SELECT_PROCESS_LSB_0;
--
SELECT_PROCESS_LSB_1: process (SELECT_I, DATA_I)
begin
	case SELECT_I (2 downto 0) is
		when "000" => DATA_LSB_1 <= DATA_I (8);
		when "001" => DATA_LSB_1 <= DATA_I (9);
		when "010" => DATA_LSB_1 <= DATA_I (10);
		when "011" => DATA_LSB_1 <= DATA_I (11);
		when "100" => DATA_LSB_1 <= DATA_I (12);
		when "101" => DATA_LSB_1 <= DATA_I (13);
		when "110" => DATA_LSB_1 <= DATA_I (14);
		when "111" => DATA_LSB_1 <= DATA_I (15);
		when others => DATA_LSB_1 <= 'X';
	end case;
end process SELECT_PROCESS_LSB_1;
--
-- MUXF7 instantiation
U_LSB_MUXF7: MUXF7
    port map (
    	I0 => DATA_LSB_0,
    	I1 => DATA_LSB_1,
    	S  => SELECT_I (3),
    	O  => DATA_LSB 
    );  
--
SELECT_PROCESS_MSB_0: process (SELECT_I, DATA_I)
begin
	case SELECT_I (2 downto 0) is
		when "000" => DATA_MSB_0 <= DATA_I (16);
		when "001" => DATA_MSB_0 <= DATA_I (17);
		when "010" => DATA_MSB_0 <= DATA_I (18);
		when "011" => DATA_MSB_0 <= DATA_I (19);
		when "100" => DATA_MSB_0 <= DATA_I (20);
		when "101" => DATA_MSB_0 <= DATA_I (21);
		when "110" => DATA_MSB_0 <= DATA_I (22);
		when "111" => DATA_MSB_0 <= DATA_I (23);
		when others => DATA_MSB_0 <= 'X';
	end case;
end process SELECT_PROCESS_MSB_0;
--
SELECT_PROCESS_MSB_1: process (SELECT_I, DATA_I)
begin
	case SELECT_I (2 downto 0) is
		when "000" => DATA_MSB_1 <= DATA_I (24);
		when "001" => DATA_MSB_1 <= DATA_I (25);
		when "010" => DATA_MSB_1 <= DATA_I (26);
		when "011" => DATA_MSB_1 <= DATA_I (27);
		when "100" => DATA_MSB_1 <= DATA_I (28);
		when "101" => DATA_MSB_1 <= DATA_I (29);
		when "110" => DATA_MSB_1 <= DATA_I (30);
		when "111" => DATA_MSB_1 <= DATA_I (31);
		when others => DATA_MSB_1 <= 'X';
	end case;
end process SELECT_PROCESS_MSB_1;
--
-- MUXF7 instantiation
U_MSB_MUXF7: MUXF7
    port map (
    	I0 => DATA_MSB_0,
    	I1 => DATA_MSB_1,
    	S  => SELECT_I (3),
    	O  => DATA_MSB
    );  
--
-- MUXF8 instantiation
U_MUXF8: MUXF8
    port map (
    	I0 => DATA_LSB,
    	I1 => DATA_MSB,
    	S  => SELECT_I (4),
    	O  => DATA_O 
    );  
--
end MUX_32_1_arch;
